// * ======================================================================
// * *                                                                    *
// * *  Copyright (c) 1996, University of Salzburg.                       *
// * *  All rights reserved.                                              *
// * *                                                                    *
// * ======================================================================
// * *                                                                    *  
// * *  DataCast Version 1.8                                              *
// * *                                                                    *
// * *  Author:  Manfred Hackel                                           *
// * *  File  :  DataCast.java                                            *
// * *  Date  :  31.01.1996                                               *
// * *                                                                    *
// * *  Description:                                                      *
// * *     Main Object                                                    *
// * *                                                                    *
// * *  Revision-History:                                                 *
// * *     1.7   Manfred Hackel   initial coding                          *
// * *     1.7R  Hilmar Linder    added RRMP support for Windows 95/NT    *
// * *     1.8   Hilmar Linder    check the OS version for RRMP support   *
// * *                            added the options -rf,-ru for RRMP      *
// * *                            added this header                       *
// * *                                                                    *
// * ======================================================================

import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.DatagramSocket;

public class DataCast extends Frame {

	private String 		language = new String(); // GUI language
	private String			transportProtocol = new String(); // transport protocol (UDP or RRMP)
   static  boolean      verbose = false;

	// pointers to other objects of the Receiver Application:
	private Receiver		receiver = null;
	private Recorder		recorder = null;
	private Repository 	repository = null;
	private UserPreferences 	userPreferences = null;

	// Menu
	MenuBar		menuBar;
	Menu			fileMenu, preferencesMenu, helpMenu;
	MenuItem 	file_start, file_stop, file_exit;
	MenuItem 	preferences_storage, preferences_view;
	Menu			preferences_filter;
	Vector		filterMenuItems = new Vector();
	MenuItem 	help_about, help_handbook;

	// Buttons	
	Button		startButton = null;
	Button		stopButton = null;
	Button		exitButton = null;

	// other GUI components:
	TextArea	messageArea = null;
	TextField	statusLine = null;
	StatusPanel	statusPanel = null;

	// Filter options
	private Vector		filterOptions = new Vector();	// which filter options are available
	private Vector		filterOptionValues = new Vector();// possible values for these filter options (vector of vectors of strings)

	// GUI strings
	String		txtErrorReadingFile = new String("Error reading file");
	String		txtStartReceiver = new String("Start receiver");
	String		txtStopReceiver = new String("Stop receiver");
	String		txtExit = new String("Exit");
	String		txtFile = new String("File");
	String		txtPreferences = new String("Preferences");
	String		txtFiltering = new String ("Filtering");
	String		txtStorage = new String("Storage");
	String		txtView = new String("View");
	String		txtHelp = new String("Help");
	String		txtAbout = new String("About");
	String		txtHandbook = new String("Handbook");
	String		txtAnnounceStart = new String("Receiver is waiting for incoming data on port");
	String		txtAnnounceStop = new String("Receiver stopped");
	String		txtTitle = new String("Internet Data Multicast Service");
	String		txtFilter1 = new String("Select the");
	String 		txtFilter2 = new String("You want to accept");
	String		txtOK = new String("OK");
	String		txtCancel = new String("Cancel");
	String		txtSelectAll = new String("Select all");
	String		txtWarning = new String("Warning!");
	String		txtNoServicesSelected1 = new String("You haven't selected any service in menu");
	String		txtNoServicesSelected2 = new String("Preferences / Filtering / Services.");
	String		txtCleaningUp = new String("cleaning up ...");
	Vector		txtFilterOptions = new Vector();
	Vector		txtFilterOptionValues = new Vector();

//	************ DataCast **********
public DataCast() {
	this("English","UDP", false);  //default
}

//	************ DataCast **********
public DataCast(String language, String transportProtocol, boolean verbose) {

	this.language = language;
	this.transportProtocol = transportProtocol;
	this.verbose = verbose;
/*
	// build menu bar:
	menuBar = new MenuBar();
	setMenuBar(menuBar);
*/
	// build first part of GUI that is needed for displaying error messages:
	Panel p = new Panel();
	p.setLayout(new GridLayout(1,1));  
	messageArea = new TextArea();
	messageArea.setEditable(false);
	p.add(messageArea);
	add("Center",p);  //Center: gets maximum space //Center

	// read GUI txts from file DataCast***.txt
	String textFileName = new String("DataCast"+language+".txt");
	File f;
	if (language.compareTo("English") != 0) {
		f = new File(textFileName);
		if (f.exists()) {
			DataInputStream d = null; 
			try {
				d = new DataInputStream(new FileInputStream(f));
				txtErrorReadingFile = d.readLine();
				txtStartReceiver = d.readLine();
				txtStopReceiver = d.readLine();
				txtExit = d.readLine();
				txtFile = d.readLine();
				txtPreferences = d.readLine();
				txtFiltering = d.readLine();
				txtStorage = d.readLine();
				txtView = d.readLine();
				txtHelp = d.readLine();
				txtAbout = d.readLine();
				txtHandbook = d.readLine();
				txtAnnounceStart = d.readLine();
				txtAnnounceStop = d.readLine();
				txtTitle = d.readLine();
				txtFilter1 = d.readLine();
				txtFilter2 = d.readLine();
				txtOK = d.readLine();
				txtCancel = d.readLine();
				txtSelectAll = d.readLine();
				txtWarning = d.readLine();
				txtNoServicesSelected1 = d.readLine();
				txtNoServicesSelected2 = d.readLine();
				txtCleaningUp = d.readLine();
			}
			catch (Exception e) {
				messageArea.appendText(txtErrorReadingFile+" "+textFileName+"\n");
			}
			finally {
				if (d != null ) {
					try { d.close(); }
					catch ( IOException ioe ) {}
				}
			}
		} 
		else {
			// else default language: English
			messageArea.appendText(txtErrorReadingFile+" "+textFileName+"\n");
		}
	}
	
	// initialise userPrefences:
	userPreferences = new UserPreferences(this, messageArea, txtErrorReadingFile);
	boolean showStatusPanel = userPreferences.showStatusPanel();
	boolean showStatusLine = userPreferences.showStatusLine();

	// read option values from file options.txt:
	f = new File("Options.txt");
	if (f.exists()) {
		DataInputStream d = null; 
		try {
			d = new DataInputStream(new FileInputStream(f));
			String s = new String();
			int optionNumber = -1;
			try {
				while (true) {
					s = d.readLine();
					if ( !s.startsWith("rem") & !s.startsWith("REM") & !(s.length() == 0) ) {
						if (s.startsWith("[")) { // begin of a new option
							optionNumber ++;
							filterOptions.addElement(s.substring(1,s.length()-1));
							Vector v = new Vector();
							filterOptionValues.addElement(v);
//							System.out.println(filterOptions.lastElement());
						}
						else { // possible value for this option
							((Vector) (filterOptionValues.elementAt(optionNumber))).addElement(s);
//							System.out.println("Option "+optionNumber+": "+s);
						}
					}
				}
			} catch (Exception e) {} // end of file
		} catch (Exception e) {
			messageArea.appendText(txtErrorReadingFile+" Options.txt\n");
		}
		finally {
			if (d != null ) {
				try { d.close(); }
				catch ( IOException ioe ) {}
			}
		}
	}
	else {
		messageArea.appendText(txtErrorReadingFile+" Options.txt\n");
	}	

	// read GUI option values from file options***.txt:
	if (language.compareTo("English") != 0) {
		textFileName = new String("Options"+language+".txt");
		f = new File(textFileName);
		if (f.exists()) {
			DataInputStream d = null;
			try {
				d = new DataInputStream(new FileInputStream(f));
				String s = new String();
				int optionNumber = -1;
				try {
					while (true) {
						s = d.readLine();
						if ( !s.startsWith("rem") & !s.startsWith("REM") & !(s.length() == 0) ) {
							if (s.startsWith("[")) { // begin of a new option
								optionNumber ++;
								txtFilterOptions.addElement(s.substring(1,s.length()-1));
								Vector v = new Vector();
								txtFilterOptionValues.addElement(v);
//								System.out.println(txtFilterOptions.lastElement());
							}
							else { // possible value for this option
								((Vector) (txtFilterOptionValues.elementAt(optionNumber))).addElement(s);
//								System.out.println("Option "+optionNumber+": "+s);
							}
						}
					}
				} catch (Exception e) {} // end of file
			} catch (Exception e) {
				// use English
			}
			finally {
				if (d != null ) {
					try { d.close(); }
					catch ( IOException ioe ) {}
				}
			}
		}
		else { // use English
			txtFilterOptions = filterOptions;
			txtFilterOptionValues = filterOptionValues;
			messageArea.appendText(txtErrorReadingFile+" "+textFileName+"\n");
		}	
	}
	else {
		txtFilterOptions = filterOptions;
		txtFilterOptionValues = filterOptionValues;
	}

	// build remaining, language-dependent GUI elements:
		// Status Line
	p = new Panel();
	p.setLayout(new GridLayout(1,1));  
	statusLine = new TextField();
	statusLine.setEditable(false);
	p.add(statusLine);
	add("North", p); //North
		// Status Panel:
	p = new Panel();
	p.setLayout(new BorderLayout());
	statusPanel = new StatusPanel();
	p.add("North",statusPanel); //North
		// Buttons:
	Panel pS = new Panel();
	startButton = new Button(txtStartReceiver);
	pS.add(startButton);
	stopButton = new Button(txtStopReceiver);
	stopButton.disable();
	pS.add(stopButton);
	exitButton = new Button(txtExit);
	pS.add(exitButton);
	p.add("South",pS); //South
	add("South",p); //South

//	// build menu bar:
	fileMenu = new Menu(txtFile);
		file_start = new MenuItem(txtStartReceiver);
		fileMenu.add(file_start);
		file_stop = new MenuItem(txtStopReceiver);
		file_stop.disable();
		fileMenu.add(file_stop);
		preferences_filter = new Menu(txtFiltering);
		fileMenu.add(preferences_filter);
		MenuItem m;
		for (int i = 0; i< filterOptions.size();++i) {
			m = new MenuItem(txtFilterOptions.elementAt(i).toString());
			preferences_filter.add(m);
		} 
		preferences_storage = new MenuItem(txtStorage);
		fileMenu.add(preferences_storage);
		preferences_view = new MenuItem(txtView);
		fileMenu.add(preferences_view);
		file_exit = new MenuItem(txtExit);
		fileMenu.add(file_exit);

	menuBar = new MenuBar();
	menuBar.add(fileMenu);
	setMenuBar(menuBar);
/*
	helpMenu = new Menu(txtHelp);
		help_about = new MenuItem(txtAbout);
		helpMenu.add(help_about);
		MenuItem help_handbook = new MenuItem(txtHandbook);
		helpMenu.add(help_handbook);
		menuBar.add(helpMenu);
		menuBar.setHelpMenu(helpMenu);
*/
}

//	************ action **********
public boolean action(Event event, Object object) {
	if ((event.target == exitButton) | (event.target ==  file_exit)) {
		startButton.disable();
		file_start.disable();
		exitButton.disable();
		file_exit.disable();
		if (receiver != null) {
			receiver.resume();
			receiver.close();
			try {
				Thread.sleep(500); 
			} catch (InterruptedException e){}
			recorder.terminate();
			recorder.close();
		}
		messageArea.appendText(txtCleaningUp+"\n");
		close();
		System.exit(0);
		return true;
	}
	if ((event.target == startButton) | (event.target == file_start)) {
		if (userPreferences.getAcceptedValues("Services").size() == 0) {
			// no services selected!
			WarningDialog d = new WarningDialog(this,txtWarning, true);
			d.init(txtNoServicesSelected1, txtNoServicesSelected2);
			d.resize(300,150);
			d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
			d.show();
			return true;
		}
		else {
 			startButton.disable();
			file_start.disable();
			exitButton.disable();
			file_exit.disable();
			preferences_filter.disable();
			preferences_storage.disable();
			if (receiver == null) {
				recorder = new Recorder(userPreferences, language, messageArea, statusLine, statusPanel);
				repository = new Repository(userPreferences, recorder, language);
				repository.checkDiskUsage();
				try {
					receiver = new Receiver(userPreferences, repository, recorder, transportProtocol);
				}
				catch (Exception e) {
					recorder.writeMessage("<Receiver.init> "+e.toString());
				}
			}
			if (receiver != null) {
				stopButton.enable();
				file_stop.enable();
				if (receiver.isAlive()) {
					receiver.resume(txtAnnounceStart+" "+userPreferences.getPortNumber()+"..");
				}
				else {
					receiver.start(txtAnnounceStart+" "+userPreferences.getPortNumber()+"...");
				}
			}
			return true;
		}
	}
	if ((event.target == stopButton) | (event.target == file_stop)) {
		receiver.suspend(txtAnnounceStop);
		exitButton.enable();
		file_exit.enable();
		startButton.enable();
		file_start.enable();
		stopButton.disable();
		file_stop.disable();
		preferences_filter.enable();
//		preferences_storage.enable();
		return true;
	}
	if (event.target == preferences_storage) {
		StorageDialog d = new StorageDialog(this,txtStorage,true);
		d.init(userPreferences,language, messageArea, txtErrorReadingFile);
		d.resize(400,250);
		d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
		d.show();
		return true;
	}
	if (event.target == preferences_view) {
		ViewDialog d = new ViewDialog(this,txtView,true);
		d.init(userPreferences,language, messageArea, txtErrorReadingFile);
		d.resize(300,200);
		d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
		d.show();
		return true;
	}
	if (event.target == help_about) {
		AboutDialog d = new AboutDialog(this,txtAbout, true);
		d.init();
		d.resize(200,170);
		d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
		d.show();
//		Point p = d.getParent().location();
//		System.out.println(p.toString());
		return true;
	}
	if (event.target == help_handbook) {

		return false;
	}
	// else
	if (event.target instanceof MenuItem) {
		if (txtFilterOptions.size() == 0) {
			txtFilterOptions = filterOptions;
			txtFilterOptionValues = filterOptionValues;
		}
		FilterDialog d = new FilterDialog(this,object.toString(),true);
		int currentOption = txtFilterOptions.indexOf(object.toString());
		if (currentOption > -1) {
			d.init(	userPreferences,
				filterOptions.elementAt(currentOption).toString(),
				object.toString(),
				(Vector) (filterOptionValues.elementAt(currentOption)), 
				(Vector) (txtFilterOptionValues.elementAt(currentOption)), 
				txtFilter1+" "+object.toString()+" "+txtFilter2,
				txtOK, txtCancel, txtSelectAll);
			d.resize(400,120+((Vector) (filterOptionValues.elementAt(currentOption))).size()*10);
			d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
			d.show();
			return true;
		}
	}
	return true;
}

//	************ handleEvent **********
public boolean handleEvent(Event evt) {  // Thomas Eder
	if (evt.id == Event.WINDOW_DESTROY) {
/*		if (receiver != null) {
			receiver.close();
		}
		close();
*/
		System.exit(0);
	}
	return super.handleEvent(evt);
}

//	************ preferredSize **********
public Dimension preferredSize() {
	return new Dimension(500, 350);
}

//	************ insets **********
public Insets insets() {
	return new Insets(60,20,10,20);
}

//	************ close **********
public void close() {
	if (receiver != null) {
		receiver.close();
		recorder.terminate();
		recorder.close();
	}
	if (userPreferences != null) {
		userPreferences.close();
	}
	if (repository != null) {
		repository.close();
	}
}

//	************ finalize **********
protected void finalize() {
	close();
}

//	************ showStatusPanel **********
public void showStatusPanel() {
	statusPanel.show();
	repaint();
}

//	************ hideStatusPanel **********
public void hideStatusPanel() {
	statusPanel.hide();
	repaint();
}

//	************ showStatusLine **********
public void showStatusLine() {
	statusLine.show();
	repaint();
}

//	************ hideStatusLine **********
public void hideStatusLine() {
	statusLine.hide();
	repaint();
}

//	************ adjustView **********
public void adjustView() {
	// adjust view to user preferences:
	if (!userPreferences.showStatusLine()) {
		statusLine.hide();
	}
	if (!userPreferences.showStatusPanel()) {
		statusPanel.hide();
	}

}

//	************ main **********
public static void main(String [] args) {
	System.out.println("DataCastReceiver V.1.8,  University of Salzburg, Jan. 1997");
	String language = new String("English");  //default GUI language
	String transportProtocol = new String("UDP"); // default transport protocol
	boolean verbose = false;
	boolean immediateStart = false;
	
	if (args.length >2) {
		System.out.println("Usage: java DataCast [-r[f|u]x] [<language>]\n");
		System.out.println("-rf ...  for the limited reliable multicast transport service (with FEC)");
		System.out.println("-ru ...  for the unreliable multicast transport service (without FEC)");
		System.out.println("-x  ...  start the DataCastReceiver immediately");
		System.out.println("-v  ...  use this for a verbose logfile");
		System.out.println("\nNote: the options -rf and -ru can only be used on Windows 95/NT platforms");
		return;
	}
	if (args.length == 2) {
		if (args[0].indexOf('u') > 0) {
			// use RRMP as transport protocol
			transportProtocol = "RRMP-unreliable";
		}
		if (args[0].indexOf('f') > 0) {
			// use RRMP as transport protocol
			transportProtocol = "RRMP-fec";
		}
		if (args[0].indexOf('x') > 0) {
			immediateStart = true;
		}
		if (args[0].indexOf('v') > 0) {
			verbose = true;
		}
		language = args[1];
	}
	if (args.length == 1) {
		if (args[0].startsWith("-")) {
			if (args[0].indexOf('u') > 0) {
				// use RRMP as transport protocol
				transportProtocol = "RRMP-unreliable";
			}
			if (args[0].indexOf('f') > 0) {
				// use RRMP as transport protocol
				transportProtocol = "RRMP-fec";
			}
			if (args[0].toLowerCase().indexOf('x') > 0) {
				immediateStart = true;
			}
			if (args[0].indexOf('v') > 0) {
				verbose = true;
			}
		}
		else { // first parameter is GUI language
			language = args[0];
			if ((language.toLowerCase().compareTo("german") == 0) | (language.toLowerCase().compareTo("deutsch") == 0)) {
				language = "Deutsch";
			}
			if ((language.toLowerCase().compareTo("italian") == 0) | (language.toLowerCase().compareTo("italiano") == 0)) {
				language = "Italiano";
			}
			if ((language.toLowerCase().compareTo("french") == 0) | (language.toLowerCase().compareTo("francais") == 0)) {
				language = "Francais";
			}
			if (	(language.toLowerCase().compareTo("spanish") == 0) | 
				(language.toLowerCase().compareTo("espanol") == 0) |
				(language.toLowerCase().compareTo("espaol") == 0)) {
					language = "Espanol";
			}
		}
	}
	DataCast isisNews = new DataCast(language, transportProtocol, verbose);
	isisNews.setTitle(isisNews.txtTitle);
	isisNews.pack();
	isisNews.show();
	isisNews.adjustView();
	if (immediateStart) {
		isisNews.postEvent(new Event(isisNews.startButton, Event.ACTION_EVENT,null));
	}
}

}

//	************ class StatusPanel **********
class StatusPanel extends Panel {
	private long receivedFiles = 0; 	// number of received files of current service 
	private long numberFiles = 0;	// total number of files expected for current service

	public StatusPanel() {
		super();
		setLayout(new GridLayout(1,0));
		validate();
	}

	public Insets insets() {
		return new Insets(0,1,0,1);
	}

	public Dimension preferredSize() {
		return new Dimension(400, 30);
	}

	public void reset(long numberFiles) {
		receivedFiles = 1;
		this.numberFiles = numberFiles;
		repaint();
	}
	public void increment() {
		receivedFiles++;
		repaint();
	}

	public void paint(Graphics g) {
		Insets panelInsets = this.insets();
		Dimension panelSize = getParent().size();
		int barLength = panelSize.width-panelInsets.left-panelInsets.right;
//		System.out.println(panelInsets.toString());
//		System.out.println(panelSize.toString());
		g.setColor(Color.black);
		g.drawRect(panelInsets.left,5,barLength,20);
		g.setColor(Color.blue);
		if (numberFiles > 0) {
			g.fillRect(panelInsets.left,5,(int) (barLength*receivedFiles/numberFiles),20);
		}
		else {
			g.setColor(this.getBackground());
			g.fillRect(panelInsets.left+1,6,barLength-2,18);
		}
	}
}