// * ======================================================================
// * *                                                                    *
// * *  Copyright (c) 1996, University of Salzburg.                       *
// * *  All rights reserved.                                              *
// * *                                                                    *
// * ======================================================================
// * *                                                                    *  
// * *  DataCast Version 1.8                                              *
// * *                                                                    *
// * *  Author:  Manfred Hackel                                           *
// * *  File  :  FilterDialog.java                                        *
// * *  Date  :  31.01.1996                                               *
// * *                                                                    *
// * *  Description:                                                      *
// * *                                                                    *
// * *                                                                    *
// * *  Revision-History:                                                 *
// * *     1.7   Manfred Hackel   initial coding                          *
// * *     1.7R  Hilmar Linder    added RRMP support for Windows 95/NT    *
// * *     1.8   Hilmar Linder    check the OS version for RRMP support   *
// * *                            added the options -rf,-ru for RRMP      *
// * *                            added this header                       *
// * *                                                                    *
// * ======================================================================

import java.awt.*;
import java.util.*;

public class FilterDialog extends Dialog {

	private Vector checkboxes;
	private UserPreferences userPreferences;
	private String option;
	private Vector values = new Vector();	// list of possible values (in English) for current option
	private Button OKButton = null;
	private Button cancelButton = null;
	private Button selectAllButton = null;

	public FilterDialog(Frame parent, String title, boolean modal) {
		super(parent, title, modal);
	}

	public void init(	UserPreferences userPref, String currentOption, String txtCurrentOption, 
			Vector optionValues, Vector txtOptionValues, String labelText,
			String txtOK, String txtCancel, String txtSelectAll) {
		// userPref = pointer to user preferences object
		// currentOption = name (in English) of the current option, e.g. "Subjects"
		// txtCurrentOption = name (in GUI language) of the current option, e.g. "Themen"
		// optionValues = list (in English) of possible values for current option , e.g. <"Politics", "Economy", ..>
		// txtOptionValues = list (in GUI language) of the current option, e.g. <"Politik", "Wirtschaft", ...>
		// labelText = text of explanatory label in this frame
		// txtOK, txtCancel, txtSelectAll = text of OK-Button, Cancel-Button and Select-All-Button in GUI language

		userPreferences = userPref;
		values = optionValues;
		option = new String(currentOption);

		Panel p;
		setLayout(new BorderLayout());

		// label at top of frame with labelText:
		p = new Panel();
		Label line1 = new Label(labelText,Label.CENTER);
		p.add(line1);
		add("North",p);

		// checkboxes for all possible values of current option:
		p = new Panel();
		checkboxes = new Vector();
		Checkbox c;
		Vector acceptedValues = new Vector();
		acceptedValues = userPreferences.getAcceptedValues(option);
		for (int i=0; i < optionValues.size();++i) {
			c = new Checkbox((String) (txtOptionValues.elementAt(i)));
			c.setState(acceptedValues.contains((String) (optionValues.elementAt(i))));
			checkboxes.addElement(c);
			p.add(c);
		}
		add("Center",p);

		// buttons at bottom of frame:
		p = new Panel();
		OKButton  = new Button(txtOK);
		p.add(OKButton);
		cancelButton = new Button(txtCancel);
		p.add(cancelButton);
		selectAllButton = new Button(txtSelectAll);
		p.add(selectAllButton);	
		add("South",p);

	}

	public boolean action(Event event, Object object) {
		if (event.target == OKButton) {
			// save settings
			Vector acceptedValues = new Vector();
			Checkbox c = new Checkbox();
			for (int i=0; i < checkboxes.size();++i) {
				c = (Checkbox) (checkboxes.elementAt(i));
				if (c.getState()) {
					acceptedValues.addElement(values.elementAt(i).toString());
				} 
			}
			userPreferences.setOptionValues(option,acceptedValues);
			dispose();
		}
		if (event.target == cancelButton) {
			dispose();
		}
		if (event.target == selectAllButton) {
			Checkbox c = new Checkbox();
			for (int i=0; i < checkboxes.size();++i) {
				c = (Checkbox) (checkboxes.elementAt(i));
				c.setState(true);
			}
		}
		return true;
	}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {
			dispose();
		}   
		return super.handleEvent(event);
	}


}