// * ======================================================================
// * *                                                                    *
// * *  Copyright (c) 1996, University of Salzburg.                       *
// * *  All rights reserved.                                              *
// * *                                                                    *
// * ======================================================================
// * *                                                                    *  
// * *  DataCast Version 1.8                                              *
// * *                                                                    *
// * *  Author:  Manfred Hackel                                           *
// * *  File  :  Segment.java                                             *
// * *  Date  :  31.01.1996                                               *
// * *                                                                    *
// * *  Description:                                                      *
// * *                                                                    *
// * *                                                                    *
// * *  Revision-History:                                                 *
// * *     1.7   Manfred Hackel   initial coding                          *
// * *     1.7R  Hilmar Linder    added RRMP support for Windows 95/NT    *
// * *     1.8   Hilmar Linder    check the OS version for RRMP support   *
// * *                            added the options -rf,-ru for RRMP      *
// * *                            added this header                       *
// * *                                                                    *
// * ======================================================================

import java.util.*;
import java.net.*;
import java.io.*;

public class Segment {
	// a Segment is the body of an UDP/IP datagram; to transmit an IsisNews document (Document.java) the document file
	// has to be split into several segments; every segment consists of a segment header and a segment payload carring some bytes of the file

	// instance variables:
	final int		datagramLength = 1024;	// must be the same as in all other DataCast classes
	final byte	protocolVersion = 1;	// if changed, change as well in class Receiver

	private int 	docID;			// document ID of the document to which this segment belongs
	private long 	seqNr;			// sequence number of the segment within the document
	public byte[] 	buffer;			// buffer containing the segment
	private boolean	isControlPacket;		// does this segment represent a control packet

//	********** init ************
public void init (byte[] buffer) throws Exception {
	// initialize the segment, i.e. analyse the header of the segment 
	// for a description of the header of a segment see IsisSender.java

	// check protocol version:
	if (buffer[0] != protocolVersion) {
		Exception e = new Exception("protocol version error");
		throw e;
	}

	// analyse flags to see if it is a control packet or a data packet
	if ((buffer[1] & 1) == 0) {
		// Control packet
		isControlPacket = true;
		if ((buffer[1] & 3) == 0) {
			// terminate receiver
			Exception e = new Exception("terminate receiver");
			throw e;
		}
	}
	else {
		// Data packet
		this.buffer = buffer;		
		isControlPacket = false;
		// get docID:
		docID = 0;
		for (int i = 2; i<4; i++) {
			if (buffer[i] < 0) {
				docID = (int) (docID *256 + buffer[i] +256);
			}
			else {
				docID = (int) (docID *256 + buffer[i]);
			}
		}
		// get sequence number of segment in document	
		seqNr = 0;
		for (int i = 4; i<8; i++) {
			if (buffer[i] < 0) {
				seqNr = (int) (seqNr *256 + buffer[i] +256);
				}
			else {
				seqNr = (int) (seqNr *256 + buffer[i]);
			}
		}
		// just for debugging:
		/*	System.out.println("Segment.docID: "+docID);
			System.out.println("Segment.seqNr: "+seqNr);
		*/
	}
}

//	********** seqNr ************
public long seqNr() {
	// returns the sequence number of the segment within the document
 	return seqNr;
}

//	********** docID ************
public int docID() {
	// returns the ID of the document to which this segment belongs
	return docID;
}

//	********** isControlPacket ************
public boolean isControlPacket() {
	return isControlPacket;
}

}