// * ======================================================================
// * *                                                                    *
// * *  Copyright (c) 1996, University of Salzburg.                       *
// * *  All rights reserved.                                              *
// * *                                                                    *
// * ======================================================================
// * *                                                                    *  
// * *  DataCast Version 1.8                                              *
// * *                                                                    *
// * *  Author:  Manfred Hackel                                           *
// * *  File  :  StorageDialog.java                                       *
// * *  Date  :  31.01.1996                                               *
// * *                                                                    *
// * *  Description:                                                      *
// * *                                                                    *
// * *                                                                    *
// * *  Revision-History:                                                 *
// * *     1.7   Manfred Hackel   initial coding                          *
// * *     1.7R  Hilmar Linder    added RRMP support for Windows 95/NT    *
// * *     1.8   Hilmar Linder    check the OS version for RRMP support   *
// * *                            added the options -rf,-ru for RRMP      *
// * *                            added this header                       *
// * *                                                                    *
// * ======================================================================

import java.awt.*;
import java.util.*;
import java.io.*;

public class StorageDialog extends Dialog {

	// instance variables:
	private UserPreferences userPreferences;
	private Checkbox displayOnScreenCheckbox;
	private Checkbox recordToFileCheckbox;
//	private TextField logFileNameField;
	private TextField maxDiskSpaceField;
//	private TextField serviceExpiryTime;
	private TextField fileExpiryTimeField;
	private TextField portNumberField;
	private TextField multicastAddrField;
	private Button OKButton = null;
	private Button cancelButton = null;
//	private Button selectAllButton = null;

	// GUI strings:
	private String txtDisplayOnScreen = new String("display log report on screen (stdout)");
	private String txtRecordToFile = new String("generate log report file");
	private String txtMaxDiskSpace = new String("maximum disk space for downloaded files in MB: ");
	private String txtFileExpiryTime1 = new String("delete downloaded files older than "); 
	private String txtFileExpiryTime2 = new String("hours at programme start");
	private String txtPortNumber = new String("UDP port number:");
	private String txtMulticastAddr = new String("Multicast Address:");
	private String txtIllegalMaxDiskSpaceValue = new String("Illegal value for maximum disk space");
	private String txtIllegalFileExpiryTimeValue = new String("Illegal value for file expiry time");
	private String txtIllegalPortNumber = new String("Illegal value for port number");
	private String txtIllegalMulticastAddr = new String("Illegal multicast address");
	private String txtWarning = new String("Warning!");
	private String txtOK = new String("OK");
	private String txtCancel = new String("Cancel");
//	private String txtSelectAll = new String("Select all");
		

	public StorageDialog(Frame parent, String title, boolean modal) {
		super(parent, title, modal);
	}

	public void init (UserPreferences userPref, String language, TextArea messageArea, String  txtErrorReadingFile) {
		// userPref = pointer to user preferences object
		// language = GUI language

		userPreferences = userPref;

		// read GUI txts from file StorageDialog***.txt
		if (language.compareTo("English") != 0) {
			String textFileName = new String("StorageDialog"+language+".txt");
			File f = new File(textFileName);
			if (f.exists()) {
				DataInputStream d = null; 
				try {
					d = new DataInputStream(new FileInputStream(f));
					txtDisplayOnScreen = d.readLine();
					txtRecordToFile = d.readLine();
					txtMaxDiskSpace = d.readLine();
					txtFileExpiryTime1 = d.readLine();
					txtFileExpiryTime2 = d.readLine();
					txtPortNumber = d.readLine();
					txtMulticastAddr = d.readLine();
					txtIllegalMaxDiskSpaceValue = d.readLine();
					txtIllegalFileExpiryTimeValue = d.readLine();
					txtIllegalPortNumber = d.readLine();
					txtIllegalMulticastAddr = d.readLine();
					txtWarning = d.readLine();
					txtOK = d.readLine();
					txtCancel = d.readLine();
//					txtSelectAll = d.readLine();
				}
				catch (Exception e) {
					messageArea.appendText(txtErrorReadingFile+" "+textFileName+"\n");	
				}
				finally {
					if (d != null ) {
						try { d.close(); }
						catch ( IOException ioe ) {}
					}
				}
			}
			else {
				 // else default language: English
				messageArea.appendText(txtErrorReadingFile+" "+textFileName+"\n");	
			}
		}

		Panel p;
		setLayout(new BorderLayout());

		// generate checkboxes and input Fields:
		p = new Panel();
		p.setLayout(new BorderLayout());
		Panel p1 = new Panel();
			recordToFileCheckbox = new Checkbox(txtRecordToFile);
			recordToFileCheckbox.setState(userPreferences.recordToFile());
			p1.add(recordToFileCheckbox);
		p.add("North",p1);
		p1 = new Panel();
			displayOnScreenCheckbox = new Checkbox(txtDisplayOnScreen);
			displayOnScreenCheckbox.setState(userPreferences.displayOnScreen());
			p1.add(displayOnScreenCheckbox);
		p.add("Center",p1);
		p1 = new Panel();
			p1.add(new Label(txtMaxDiskSpace));
			String maxDiskSpace = String.valueOf(userPreferences.getMaxDiskSpace());
			maxDiskSpaceField = new TextField(maxDiskSpace,4);
			p1.add(maxDiskSpaceField);		
			p.add("South",p1);
		add("North",p);
		p = new Panel();
		p.setLayout(new BorderLayout());
		p1 = new Panel();
			p1.add(new Label(txtFileExpiryTime1));
			String fileExpiryTime = String.valueOf(userPreferences.getFileExpiryTime());
			fileExpiryTimeField = new TextField(fileExpiryTime,4);
			p1.add(fileExpiryTimeField);
			p1.add(new Label(txtFileExpiryTime2));
			p.add("North",p1);
		p1 = new Panel();
			p1.add(new Label(txtPortNumber));
			String portNumber = String.valueOf(userPreferences.getPortNumber());
			portNumberField = new TextField(portNumber,6);
			p1.add(portNumberField);
			p.add("Center",p1);
		p1 = new Panel();
			p1.add(new Label(txtMulticastAddr));
			String multicastAddr = String.valueOf(userPreferences.getMulticastAddr());
			multicastAddrField = new TextField(multicastAddr,15);
			p1.add(multicastAddrField);
			p.add("South",p1);
		add("Center",p);	

		// buttons at bottom of frame:
		p = new Panel();
		OKButton  = new Button(txtOK);
		p.add(OKButton);
		cancelButton = new Button(txtCancel);
		p.add(cancelButton);
//		selectAllButton = new Button(txtSelectAll);
//		p.add(selectAllButton);	
		add("South",p);

	}

	public boolean action(Event event, Object object) {
		if (event.target == OKButton) {
			// save settings
			if (displayOnScreenCheckbox.getState()) {
				userPreferences.enableDisplayOnScreen();
			}
			else {
				userPreferences.disableDisplayOnScreen();
			}
			if (recordToFileCheckbox.getState()) {
				userPreferences.enableRecordToFile();
			}
			else {
				userPreferences.disableRecordToFile();
			}
			try {
				userPreferences.setMaxDiskSpace(Integer.parseInt(maxDiskSpaceField.getText()));
			}
			catch(Exception e) {
				WarningDialog d = new WarningDialog((Frame) this.getParent(),txtWarning,true);
				d.init(txtIllegalMaxDiskSpaceValue);
				d.resize(250,100);
				d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
				d.show();
				return true;
			}
			try {
				userPreferences.setFileExpiryTime(Integer.parseInt(fileExpiryTimeField.getText()));
			}
			catch(Exception e) {
				WarningDialog d = new WarningDialog((Frame) this.getParent(),txtWarning,true);
				d.init(txtIllegalFileExpiryTimeValue);
				d.resize(250,100);
				d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
				d.show();
				return true;
			}
			try {
				userPreferences.setPortNumber(Integer.parseInt(portNumberField.getText()));
			}
			catch(Exception e) {
				WarningDialog d = new WarningDialog((Frame) this.getParent(),txtWarning,true);
				d.init(txtIllegalPortNumber);
				d.resize(250,100);
				d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
				d.show();
				return true;
			}
			try {
				userPreferences.setMulticastAddr(multicastAddrField.getText());
			}
			catch(Exception e) {
				WarningDialog d = new WarningDialog((Frame) this.getParent(),txtWarning,true);
				d.init(txtIllegalMulticastAddr);
				d.resize(250,100);
				d.move((d.getParent().size().width-d.size().width)/2,(d.getParent().size().height-d.size().height)/2);
				d.show();
				return true;
			}
			dispose();
		}
		if (event.target == cancelButton) {
			dispose();
		}
/*		if (event.target == selectAllButton) {
		}
*/		return true;
	}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {
			dispose();
		}   
		return super.handleEvent(event);
	}

}