// * ======================================================================
// * *                                                                    *
// * *  Copyright (c) 1996, University of Salzburg.                       *
// * *  All rights reserved.                                              *
// * *                                                                    *
// * ======================================================================
// * *                                                                    *  
// * *  DataCast Version 1.8                                              *
// * *                                                                    *
// * *  Author:  Manfred Hackel                                           *
// * *  File  :  ViewDialog.java                                          *
// * *  Date  :  31.01.1996                                               *
// * *                                                                    *
// * *  Description:                                                      *
// * *                                                                    *
// * *                                                                    *
// * *  Revision-History:                                                 *
// * *     1.7   Manfred Hackel   initial coding                          *
// * *     1.7R  Hilmar Linder    added RRMP support for Windows 95/NT    *
// * *     1.8   Hilmar Linder    check the OS version for RRMP support   *
// * *                            added the options -rf,-ru for RRMP      *
// * *                            added this header                       *
// * *                                                                    *
// * ======================================================================

import java.awt.*;
import java.util.*;
import java.io.*;

public class ViewDialog extends Dialog {

	// instance variables:
	private UserPreferences userPreferences;
	private Checkbox detailedErrorMessagesCheckbox;
	private Checkbox showStatusLineCheckbox;
	private Checkbox showStatusPanelCheckbox;
	private Button OKButton = null;
	private Button cancelButton = null;
	private Button selectAllButton = null;


	public ViewDialog(Frame parent, String title, boolean modal) {
		super(parent, title, modal);
	}

	public void init (UserPreferences userPref, String language, TextArea messageArea, String  txtErrorReadingFile) {
		// userPref = pointer to user preferences object
		// language = GUI language

		// GUI strings:
		String txtDetailedErrorMessages = new String("display detailed error messages");
		String txtShowStatusLine = new String("show status line with arriving file names");
		String txtShowStatusPanel = new String("show status panel"); 
		String txtOK = new String("OK");
		String txtCancel = new String("Cancel");
		String txtSelectAll = new String("Select all");
		
		userPreferences = userPref;

		// read GUI txts from file ViewDialog***.txt
		if (language.compareTo("English") != 0) {
			String textFileName = new String("ViewDialog"+language+".txt");
			File f = new File(textFileName);
			if (f.exists()) {
				DataInputStream d = null; 
				try {
					d = new DataInputStream(new FileInputStream(f));
					txtDetailedErrorMessages = d.readLine();
					txtShowStatusLine = d.readLine();
					txtShowStatusPanel = d.readLine();
					txtOK = d.readLine();
					txtCancel = d.readLine();
					txtSelectAll = d.readLine();
				}
				catch (Exception e) {
					messageArea.appendText(txtErrorReadingFile+" "+textFileName);	
				}
				finally {
					if (d != null ) {
						try { d.close(); }
						catch ( IOException ioe ) {}
					}
				}
			}
			else {
				 // else default language: English
				messageArea.appendText(txtErrorReadingFile+" "+textFileName);	
			}
		}

		Panel p;
		setLayout(new BorderLayout());

		// generate checkboxes 
		p = new Panel();
		detailedErrorMessagesCheckbox = new Checkbox(txtDetailedErrorMessages);
			detailedErrorMessagesCheckbox.setState(userPreferences.detailedErrorMessages());
			p.add(detailedErrorMessagesCheckbox);
		showStatusLineCheckbox = new Checkbox(txtShowStatusLine);
			showStatusLineCheckbox.setState(userPreferences.showStatusLine());
			p.add(showStatusLineCheckbox);
		showStatusPanelCheckbox = new Checkbox(txtShowStatusPanel);
			showStatusPanelCheckbox .setState(userPreferences.showStatusPanel());
			p.add(showStatusPanelCheckbox );
		add("Center",p);

		// buttons at bottom of frame:
		p = new Panel();
		OKButton  = new Button(txtOK);
		p.add(OKButton);
		cancelButton = new Button(txtCancel);
		p.add(cancelButton);
		selectAllButton = new Button(txtSelectAll);
		p.add(selectAllButton);	
		add("South",p);

	}

	public boolean action(Event event, Object object) {
		if (event.target == OKButton) {
			// save settings
			userPreferences.setDetailedErrorMessages(detailedErrorMessagesCheckbox.getState());
			userPreferences.setShowStatusLine(showStatusLineCheckbox.getState());
			userPreferences.setShowStatusPanel(showStatusPanelCheckbox.getState());
			dispose();
		}
		if (event.target == cancelButton) {
			dispose();
		}
		if (event.target == selectAllButton) {
			detailedErrorMessagesCheckbox.setState(true);
			showStatusLineCheckbox.setState(true);
			showStatusPanelCheckbox.setState(true);
		}
		return true;
	}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {
			dispose();
		}   
		return super.handleEvent(event);
	}


}